const utils = require('../utils/utils.js')

module.exports = {
  name: 'build',
  alias: ['b'],
  run: async toolbox => {
    const {
      parameters,
      template: { mobile },
      print: { info, success, error, table, spin },
      filesystem: fs,
      system: { exec }
    } = toolbox

    let options = await getOptions(parameters, fs, error)
    await logCommand(options, info)
    let { platform, isPrintOnlyMode } = options

    // Pre-build
    fs.dir('release') // Create directory if it does not exist fs-jetpack

    // Arch Build
    if (platform === 'ios') {
      await buildIos(options, fs, true)
      if (isPrintOnlyMode) return
      await utils.pauseForATime(2000)
      await buildIos(options, fs, isPrintOnlyMode)
    } else if (platform === 'android') {
      if (!options.isPrintOnlyMode) {
        options.isPrintOnlyMode = true
        await buildAndroidAPKs(options, fs, error)
        await utils.pauseForATime(2000)
        options.isPrintOnlyMode = false
      }
      await buildAndroidAPKs(options, fs, error)
    }

    // Done With Build
    if (!options.jenkins) await utils.spawnAndLog('say all done')
    info(' ')
  }
}

let logCommand = async (options, info) => {
  let command = ['ontrac', 'build']
  for (let i in options) {
    if (!options.hasOwnProperty(i)) continue
    command.push('--' + i)
    if (options[i] === true) continue
    command.push(options[i])
  }
  info(command.join(' '))
  info(' ')
  await utils.pauseForATime(2000)
}

let buildIos = async (options, fs, isPrintOnlyMode) => {
  let { removePlatform } = options
  if (removePlatform) await utils.platform('ios', options, isPrintOnlyMode)
  await utils.validateIosFilesAndUpdate(fs, isPrintOnlyMode)
  await utils.autoSign(isPrintOnlyMode)
  await utils.red5Fix(fs, isPrintOnlyMode)
  await utils.pushNotificationEntitlementFix(fs, isPrintOnlyMode)
  await utils.preventUpsideDown(fs, isPrintOnlyMode)
  await utils.build('ios', isPrintOnlyMode)
  await utils.copyIpaToReleaseFolder(isPrintOnlyMode, 'adhoc', fs)
  if (!options.jenkins) await utils.openIos(isPrintOnlyMode)
  else {
    await utils.build('ios', isPrintOnlyMode, true)
    await utils.copyIpaToReleaseFolder(isPrintOnlyMode, 'distrib', fs)
  }
  if (isPrintOnlyMode) console.log('\n*****************************\n')
}

let buildAndroidAPKs = async (options, fs) => {
  let { keystore, keystorePassword, arch, isPrintOnlyMode } = options
  await utils.validateKeystore(keystore, keystorePassword, isPrintOnlyMode)

  if (arch === 'both') {
    await createAndroidBuild('armv7', options, fs, isPrintOnlyMode)
    await createAndroidBuild('arm64', options, fs, isPrintOnlyMode)
    return
  }
  await createAndroidBuild(arch, options, fs)
}

let createAndroidBuild = async (arch, options, fs) => {
  let { platform, keystore, keystorePassword, isPrintOnlyMode } = options
  await utils.updateArch(arch, fs, isPrintOnlyMode)
  if (options.removePlatform) await utils.platform(platform, options)
  await utils.build('android', isPrintOnlyMode)
  await utils.copyApkToReleaseFolder(arch, isPrintOnlyMode)
  await utils.sign(keystore, keystorePassword, arch, isPrintOnlyMode)
  await utils.zipalign(arch, keystore, isPrintOnlyMode)
  if (!options.jenkins) await utils.openAndroid(isPrintOnlyMode)
  if (isPrintOnlyMode) console.log('\n*****************************\n')
}

let getOptions = async (parameters, fs, error) => {
  // parameters.options comes in as a JSON object with ONLY strings for values. Thus if someone types false we will get "false"
  let {
    keystore,
    keystorePassword,
    arch,
    isPrintOnlyMode,
    platform,
    removePlatform,
    jenkins
  } = parameters.options

  if (jenkins) {
    console.log('Building for Jenkins')
    jenkins = true
  }

  if (!isPrintOnlyMode) {
    isPrintOnlyMode = await utils.ask('Boolean', 'Only print output')
  }
  if (!removePlatform) {
    removePlatform = await utils.ask('Boolean', 'Remove platform')
  }
  if (!platform) {
    platform = await utils.ask('Select', 'Select a platform', [
      'ios',
      'android'
    ])
  }

  if (platform === 'android' && !arch) {
    if (!utils.listOfKeystore(await fs.listAsync('keystore'))) {
      return error('No keystores')
    }
    const keystores = utils.listOfKeystore(await fs.listAsync('keystore'))
    arch = await utils.ask('Select', 'Select a arch', [
      'both',
      'armv7',
      'arm64'
    ])
    if (!keystore) {
      keystore = await utils.ask('Select', 'Select a keystore', keystores)
    }
    if (!keystorePassword) {
      keystorePassword = await utils.ask(
        'Password',
        'What is the keystore password',
        keystores
      )
    }
  }
  let opts = {
    keystorePassword,
    keystore,
    arch,
    isPrintOnlyMode,
    platform,
    removePlatform,
    jenkins
  }
  for (let i in opts) {
    if (opts[i] === undefined) delete opts[i]
    // Force to boolean if they were string "false"
    if (opts[i] === 'false') opts[i] = false
  }
  return opts
}
