const editJsonFile = require('edit-json-file')
const packageBaseJson = editJsonFile(`./package.base.json`)

const templates = './templates'
const sep = require('path').sep
const path = require('path')
const { Select, Input, Password } = require('enquirer')
const spawn = require('child_process').spawn
const { exec } = require('child_process')
const _fs = require('fs')
const convert = require('xml-js')
const fileOptions = { compact: true, spaces: 4 }

const listOfKeystore = dir => {
  return dir.map(item => item.replace('.keystore', ''))
}

let convertToArray = str => {
  if (str && str.split) {
    return str.split(' ')
  }
  return str
}

let conversSlashes = args => {
  for (let i in args) {
    args[i] = args[i].replace(/\//g, sep)
  }
  return args
}

let spawnAndLog = (cmd, isPrintOnlyMode, onStdOut) => {
  return new Promise((resolve, reject) => {
    cmd = convertToArray(cmd)
    _log(`${cmd.join(' ')} `)
    let program = cmd[0]
    let args = cmd.slice(1)
    if (isPrintOnlyMode) return resolve()
    let res = spawn(program, args, { shell: true })
    res.on('exit', resolve)
    res.stderr.on('data', _error)
    res.stdout.on('data', data => {
      let dt = String(data)
      if (onStdOut) onStdOut(dt)
      _success(dt)
    })
  })
}

const ask = async (type, message, choices) => {
  let Constructors = {
    Select: Select,
    Input: Input,
    Password: Password,
    Boolean: Select
  }
  if (type === 'Boolean') choices = ['No', 'Yes']
  let question = new Constructors[type]({
    name: Math.random() / 1000,
    message,
    choices
  })
  let response = await question.run()
  if (type === 'Boolean') response = convertBooleanResponse(response)
  return response
}

let convertBooleanResponse = val => {
  return val === 'Yes' || val === 'true' || val === true
}

const yarnBuild = async () => {
  await spawnAndLog('yarn build')
}

const platform = async (platform, options, isPrintOnlyMode) => {
  let commands = ['cordova platform rm ios', 'cordova platform rm android']
  commands.push('rm -rf plugins/*')
  if (platform === 'ios') platform = 'ios@5.1.1'
  else if (platform === 'android') platform = 'android@8.1.0'
  commands.push('cordova platform add ' + platform)
  for (let i in commands) {
    await spawnAndLog(commands[i], options.isPrintOnlyMode || isPrintOnlyMode)
  }
  return commands.join('\n')
}

const validateKeystore = async (keystore, password, isPrintOnlyMode) => {
  await spawnAndLog(
    `keytool -list -v -keystore ./keystore/${keystore}.keystore -storepass ${password}`,
    isPrintOnlyMode,
    data => {
      if (data.includes('password was incorrect')) {
        throw _error('Incorrect Password')
      }
    }
  )
}

const build = async (platform, isPrintOnlyMode, forceIosAppStore = false) => {
  let args = ['build', platform]
  if (platform === 'android') args.push('--release')
  else if (platform === 'ios') args.push('--release --device')
  if (forceIosAppStore) args.push('--packageType="app-store"')
  let command = 'cordova ' + args.join(' ')
  await spawnAndLog(command, isPrintOnlyMode)
  return command
}

const cleanReleaseFolder = async isPrintOnlyMode => {
  let command = 'rm release/*'
  await spawnAndLog(command, isPrintOnlyMode)
  return command
}

const openIos = async isPrintOnlyMode => {
  let command = 'open platforms/ios/' + getProjectName(true) + '.xcworkspace/'
  await spawnAndLog(command, isPrintOnlyMode)
  return command
}

const openAndroid = async isPrintOnlyMode => {
  let command = 'open ./release'
  await spawnAndLog(command, isPrintOnlyMode)
  return command
}

const copyApkToReleaseFolder = async (arch, isPrintOnlyMode) => {
  let command = `cp platforms/android/app/build/outputs/apk/${arch}/release/app-${arch}-release-unsigned.apk ./release/`
  await spawnAndLog(command, isPrintOnlyMode)
  return command
}

const copyIpaToReleaseFolder = async (isPrintOnlyMode, postfix, fs) => {
  let version = getVersionNumber()
  let returnMessage = `copy ipa to release folder with version: ${version} and postfix: ${postfix}`
  console.log(returnMessage)
  if (isPrintOnlyMode) return returnMessage
  let projectShortName = getProjectName()
  fs.copy(
    `./platforms/ios/build/device/${projectShortName}.ipa`,
    `./release/${projectShortName}_${version}_${postfix}.ipa`
  )

  return returnMessage
}

const updateVersion = async (version, timestamp, fs) => {
  const androidVersion = String(timestamp).split('.')[0]
  if (JSON.stringify(packageBaseJson.data) === '{}') {
    throw _error('INVALID PACKAGE.BASE.JSON')
  }
  packageBaseJson.set('version', version)
  await fs.file('package.json', {
    content: packageBaseJson.data
  })

  const xml = fs.read(`.${sep}config.base.xml`)
  let result = convert.xml2js(xml, fileOptions)
  result.widget._attributes['android-versionCode'] = androidVersion
  result.widget._attributes.version = version

  const updatedFile = convert.json2xml(result, fileOptions)
  fs.file('config.xml', { content: updatedFile })
}

const updateArch = async (arch, fs, isPrintOnlyMode) => {
  if (isPrintOnlyMode) return
  const xml = fs.read(`.${sep}config.xml`)
  let result = convert.xml2js(xml, fileOptions)

  result.widget.preference.map(item => {
    if (item._attributes.name === 'buildArchitecture') {
      item._attributes.value = arch || 'arm64'
    }
  })

  const updatedFile = convert.json2xml(result, fileOptions)
  fs.file('config.xml', { content: updatedFile })
}

const updatePackageJson = async (CLIENT, fs) => {
  let packageJson = editJsonFile(`./package.json`)
  const config = fs.read(`${templates}${sep}${CLIENT}${sep}config.json`, 'json')
  packageJson.set('name', config.name)
  packageJson.set('displayName', config.displayName)
  packageJson.set('description', config.description)
  packageJson.set('uri', config.URI)
  packageJson.set('company_id', config.company_id)
  packageJson.set('deploy.production.path', config.serverPath)
  packageJson.set('cordova.plugins.cordova-plugin-facebook4', config.facebook)
  packageJson.set('paymentMethodUrl', config.paymentMethodUrl)
  packageJson.data = replaceAllMatches(packageJson.data, config)
  await fs.file('package.json', {
    content: packageJson.data
  })
}

const updateIndexHtml = (CLIENT, fs) => {
  const config = fs.read(`${templates}${sep}${CLIENT}${sep}config.json`, 'json')
  let indexString = fs.read('./public/index.html')
  indexString = indexString.replace(
    /<title>.*<\/title>/g,
    `<title>${config.displayName}</title>`
  )
  fs.file('public/index.html', {
    content: indexString
  })
}

const replaceAllMatches = (configXMLinJSON, configJSON) => {
  let str = JSON.stringify(configXMLinJSON, null, 2)
  for (let i in configJSON) {
    str = str.replace(`{{${i}}}`, configJSON[i])
  }
  return JSON.parse(str)
}

const updateConfigXml = (CLIENT, fs) => {
  const config = fs.read(`${templates}${sep}${CLIENT}${sep}config.json`, 'json')

  const xml = fs.read(`.${sep}config.xml`)
  let result = convert.xml2js(xml, fileOptions)
  result.widget._attributes.id = config.id
  result.widget.name._text = config.displayName
  result.widget.description._text = config.description

  result.widget.plugin.map(item => {
    if (item._attributes.name === 'cordova-plugin-facebook4') {
      item.variable[0]._attributes.value = config.facebook.APP_ID
      item.variable[1]._attributes.value = config.facebook.APP_NAME
    }
  })
  result = replaceAllMatches(result, config)
  const updatedFile = convert.json2xml(result, fileOptions)

  fs.file('config.xml', { content: updatedFile })
}

const removeAndCopy = (fs, pathToKeep, pathToRemove) => {
  let exists = fs.exists(pathToKeep)
  if (!exists) return _error('NO FILES TO COPY ' + pathToKeep)
  fs.remove(pathToRemove)
  fs.copy(pathToKeep, pathToRemove, { overwrite: true })
}

const deleteAndCopyFiles = (CLIENT, fs) => {
  const commonDir = `${templates}${sep}_common${sep}`
  const templateDir = `${templates}${sep}${CLIENT}${sep}`

  removeAndCopy(fs, `${templateDir}images`, `src${sep}images`)
  removeAndCopy(fs, `${templateDir}fonts`, `src${sep}fonts`)
  removeAndCopy(fs, `${templateDir}res`, `res`)
  removeAndCopy(fs, `${templateDir}pages.json`, `src${sep}pages.json`)
  removeAndCopy(
    fs,
    `${templateDir}custom.scss`,
    `src${sep}css${sep}custom.scss`
  )
  removeAndCopy(
    fs,
    `${templateDir}res${sep}store${sep}icons${sep}icon-1024.jpg`,
    `res${sep}icons${sep}ios${sep}icon-1024.png`
  )
  removeAndCopy(
    fs,
    `${templateDir}res${sep}splash.png`,
    `res${sep}screens${sep}ios${sep}Default@2x~universal~anyany.png`
  )

  // Safe checking if new structure works
  if (_fs.existsSync(commonDir)) {
    removeAndCopy(
      fs,
      `${commonDir}google-services.json`,
      `nativeConfig${sep}google-services.json`
    )
  } else {
    removeAndCopy(
      fs,
      `${templateDir}google-services.json`,
      `nativeConfig${sep}google-services.json`
    )
  }
}

const sign = async (keystore, keystorePassword, arch, isPrintOnlyMode) => {
  if (keystore === 'Vendoti') keystore = 'ontrac'
  let command = `jarsigner -verbose -storepass ${keystorePassword} -keypass ${keystorePassword} -sigalg SHA1withRSA -digestalg SHA1 -keystore ./keystore/${keystore}.keystore ./release/app-${arch}-release-unsigned.apk ${keystore}`
  await spawnAndLog(command, isPrintOnlyMode)
  return command
}

const zipalign = async (arch, appName, isPrintOnlyMode) => {
  let version = getVersionNumber()
  let commands = [
    `zipalign -f -v 4 ./release/app-${arch}-release-unsigned.apk ./release/${appName}_${version}_${arch}.apk`,
    `rm ./release/app-${arch}-release-unsigned.apk`
  ]
  for (let i in commands) {
    await spawnAndLog(commands[i], isPrintOnlyMode)
  }
  return commands.join('\n')
}

const pm2 = async (env, client, version) => {
  if (env === 'sandbox') env = 'sand'
  if (env === 'production') env = 'prod'
  let cmd = ['deploy', env, 'update']
  if (client === 'all') {
    await _runPm2(
      cmd.concat(['&&', 'pm2', 'exec', 'deploy', env, 'exec', './build.sh'])
    )
  } else {
    await _runPm2(
      cmd.concat([
        '&&',
        'pm2',
        'deploy',
        env,
        'exec',
        `"./buildOne.sh ${client}  ${version} ${env} "`
      ])
    )
  }
}

const _runPm2 = async commands => {
  await spawnAndLog(['pm2'].concat(commands))
}

const validateIosFilesAndUpdate = async (fs, isPrintOnlyMode) => {
  if (isPrintOnlyMode) return
  const cordovaFilePath = `platforms${sep}ios${sep}cordova${sep}lib${sep}list-emulator-build-targets`
  const originalFile = _fs.readFileSync(
    `${__dirname}${sep}build-emulator`,
    'utf-8'
  )
  const cordovaFile = _fs.readFileSync(cordovaFilePath, 'utf-8')
  if (cordovaFile === originalFile) {
    await fs.copyAsync(
      `${__dirname}${sep}build-emulator-fix`,
      cordovaFilePath,
      {
        overwrite: true
      }
    )
  }
}
const red5Fix = async (fs, isPrintOnlyMode) => {
  if (isPrintOnlyMode) return
  const pattern = /.*\/\* R5Streaming.framework \*\/ = {isa = PBXFileReference.*/gm
  const projectName = getProjectName()
  const filePath = `platforms${sep}ios${sep}${projectName}.xcodeproj${sep}project.pbxproj`
  await replaceInFile(pattern, filePath, fs)
}

const pushNotificationEntitlementFix = async (fs, isPrintOnlyMode) => {
  if (isPrintOnlyMode) return
  const replaceWith = `<dict>
    <key>aps-environment</key>
    <string>development</string>`
  const projectName = getProjectName()
  const filePath = path.join(
    process.cwd(),
    `platforms/ios/${projectName}/Resources/${projectName}.entitlements`
  )
  _log(filePath)
  await replaceInFile(/<key>aps-environment<\/key>/g, filePath, fs, '')
  await replaceInFile(/<string>development<\/string>/g, filePath, fs, '')
  await replaceInFile(/<dict>/g, filePath, fs, replaceWith)
}

const preventUpsideDown = async (fs, isPrintOnlyMode) => {
  if (isPrintOnlyMode) return
  let projectName = getProjectName()
  let filePath = `platforms/ios/${projectName}/${projectName}-Info.plist`
  const pattern = /<string>UIInterfaceOrientationPortraitUpsideDown<\/string>/gm
  await replaceInFile(pattern, filePath, fs)
}

const getProjectName = escape => {
  let packageJson = editJsonFile('./package.json')
  let projectName = packageJson.data.displayName
  if (!escape) return projectName
  return projectName.replace(/ /g, '\\ ')
}

const getProjectShortName = () => {
  let packageJson = editJsonFile('./package.json')
  let projectShortName = packageJson.data.name
  return projectShortName
}

const getVersionNumber = () => {
  let packageJson = editJsonFile('./package.json')
  let version = packageJson.data.version
  return version
}

const autoSign = isPrintOnlyMode => {
  if (isPrintOnlyMode) return
  let projectName = getProjectName()
  let xcode = require('xcode')

  let fs = require('fs')

  let projectPath = path.join(
    process.cwd(),
    './platforms/ios/' + projectName + '.xcodeproj/project.pbxproj'
  )

  let myProj = xcode.project(projectPath)
  return new Promise(resolve => {
    myProj.parse(function(err) {
      if (err) return _log(JSON.stringify(err))
      myProj.pbxXCBuildConfigurationSection()[
        '1D6058950D05DD3E006BFB54'
      ].buildSettings.CODE_SIGN_IDENTITY = '"Apple Development"'
      myProj.pbxXCBuildConfigurationSection()[
        '1D6058950D05DD3E006BFB54'
      ].buildSettings.CODE_SIGN_STYLE = 'Automatic'
      myProj.pbxXCBuildConfigurationSection()[
        '1D6058950D05DD3E006BFB54'
      ].buildSettings.COPY_PHASE_STRIP = 'YES'
      myProj.pbxXCBuildConfigurationSection()[
        '1D6058950D05DD3E006BFB54'
      ].buildSettings.DEVELOPMENT_TEAM = 'QQPD9BB4K6'
      myProj.pbxXCBuildConfigurationSection()[
        '1D6058950D05DD3E006BFB54'
      ].buildSettings.PROVISIONING_PROFILE_SPECIFIER = '""'
      fs.writeFileSync(projectPath, myProj.writeSync())
      resolve()
    })
  })
}

const replaceInFile = async (pattern, filePath, fs, replaceWith) => {
  let file = fs.read(filePath)
  const regex = new RegExp(pattern)
  if (!file) return _error('DID NOT FIX FILE ' + filePath + ' ' + pattern)
  if (file.match(regex)) {
    file = file.replace(pattern, replaceWith || '')
    await fs.writeAsync(filePath, file)
  }
}

const copyBaseFiles = async fs => {
  await fs.copyAsync(`config.base.xml`, `config.xml`, {
    overwrite: true
  })
  await fs.copyAsync(`package.base.json`, `package.json`, {
    overwrite: true
  })
}

let pauseForATime = time => {
  return new Promise((resolve, reject) => {
    setTimeout(resolve, time)
  })
}

const _success = message => {
  console.log('\x1b[32m', String(message))
}

const _error = message => {
  console.log('\x1b[31m', String(message))
}

const _log = message => {
  console.log('\x1b[36m', String(message))
}

module.exports = {
  listOfKeystore,
  ask,
  yarnBuild,
  platform,
  build,
  copyApkToReleaseFolder,
  copyIpaToReleaseFolder,
  updateVersion,
  updateArch,
  sign,
  zipalign,
  pm2,
  updatePackageJson,
  updateIndexHtml,
  updateConfigXml,
  deleteAndCopyFiles,
  validateIosFilesAndUpdate,
  validateKeystore,
  copyBaseFiles,
  red5Fix,
  spawnAndLog,
  pauseForATime,
  preventUpsideDown,
  autoSign,
  pushNotificationEntitlementFix,
  openIos,
  openAndroid,
  cleanReleaseFolder
}
